"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initLogger = exports.errorLogger = exports.fatalLogger = exports.mainLogger = void 0;
const log4js = require("log4js");
const fs_1 = require("fs");
const log4jsExtend = require('log4js-extend'); // eslint-disable-line @typescript-eslint/no-var-requires
class Logger {
    constructor(baseDir, config) {
        this.baseDir = baseDir;
        this.config = config;
        this.logDir = '';
        this.log4Conf = null;
        this._setupLogs();
    }
    // Checks if the configuration has the required components
    _checkValidConfig() {
        const config = this.config;
        if (!config.dir)
            throw Error('Fatal Error: Log directory not defined.');
        if (!config.files || typeof config.files !== 'object')
            throw Error('Fatal Error: Valid log file locations not provided.');
    }
    // Add filenames to each appender of type 'file'
    _addFileNamesToAppenders() {
        const conf = this.log4Conf;
        for (const key in conf.appenders) {
            const appender = conf.appenders[key]; // eslint-disable-line security/detect-object-injection
            if (appender.type !== 'file')
                continue;
            appender.filename = `${this.logDir}/${key}.log`;
        }
    }
    _configureLogs() {
        return log4js.configure(this.log4Conf);
    }
    // Get the specified logger
    getLogger(logger) {
        return log4js.getLogger(logger);
    }
    // Setup the logs with the provided configuration using the base directory provided for relative paths
    _setupLogs() {
        const baseDir = this.baseDir;
        const config = this.config;
        if (!baseDir)
            throw Error('Fatal Error: Base directory not defined.');
        if (!config)
            throw Error('Fatal Error: No configuration provided.');
        this._checkValidConfig();
        // Makes specified directory if it doesn't exist
        if (config.dir) {
            const allArchiversLogDir = `${baseDir}/${config.dir.split('/')[0]}`;
            this.getLogger('main').info('allArchiversLogDir', allArchiversLogDir);
            if (!(0, fs_1.existsSync)(allArchiversLogDir))
                (0, fs_1.mkdirSync)(allArchiversLogDir); // eslint-disable-line security/detect-non-literal-fs-filename
        }
        this.logDir = `${baseDir}/${config.dir}`;
        if (!(0, fs_1.existsSync)(this.logDir))
            (0, fs_1.mkdirSync)(this.logDir); // eslint-disable-line security/detect-non-literal-fs-filename
        // Read the log config from log config file
        this.log4Conf = config.options;
        log4jsExtend(log4js);
        this._addFileNamesToAppenders();
        this._configureLogs();
        this.getLogger('main').info('Logger initialized.');
    }
    // Tells this module that the server is shutting down, returns a Promise that resolves when all logs have been written to file, sockets are closed, etc.
    shutdown() {
        return new Promise((resolve) => {
            log4js.shutdown(() => {
                resolve('done');
            });
        });
    }
}
function initLogger(baseDir, logsConfig) {
    const logger = new Logger(baseDir, logsConfig);
    exports.mainLogger = logger.getLogger('main');
    exports.fatalLogger = logger.getLogger('fatal');
    exports.errorLogger = logger.getLogger('errorFile');
}
exports.initLogger = initLogger;
exports.default = Logger;
//# sourceMappingURL=Logger.js.map