"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cycleCheckpointManager = exports.CycleRadixDigestTally = exports.CycleCheckpointBucket = exports.CycleCheckpointRadixDigest = exports.CycleCheckpointRadixEntry = exports.calculateBucketID = exports.CycleCheckpointData = void 0;
const CheckpointData_1 = require("./CheckpointData");
const Crypto = require("../Crypto");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Logger = require("../Logger");
const cycles_1 = require("../dbstore/cycles");
//Represents a single piece of cycle data
class CycleCheckpointData extends CheckpointData_1.CheckpointData {
    constructor(cycle) {
        const cycleHash = Crypto.hash(lib_types_1.Utils.safeStringify(cycle)).toLowerCase();
        super(cycleHash.substring(0, 2), // address (first 2 chars)
        cycle.cycleRecord.start, // timestamp from cycleRecord
        cycleHash, // hash
        CheckpointData_1.CheckpointType.Cycle, // class type 0 for cycle
        cycle // data
        );
    }
}
exports.CycleCheckpointData = CycleCheckpointData;
function calculateBucketID(cycle) {
    if (!cycle || cycle.counter === undefined) {
        Logger.mainLogger.error('Invalid cycle data');
        throw new Error('Invalid cycle data');
    }
    return cycle.counter.toString();
}
exports.calculateBucketID = calculateBucketID;
//Represents a single radix entry in a bucket
class CycleCheckpointRadixEntry extends CheckpointData_1.CheckpointRadixEntry {
    constructor(radix) {
        super(radix);
    }
}
exports.CycleCheckpointRadixEntry = CycleCheckpointRadixEntry;
//Represents a single radix entry in a bucket
class CycleCheckpointRadixDigest extends CheckpointData_1.CheckpointRadixDigest {
    constructor(radix, hash, itemCount) {
        super(radix, hash, itemCount);
    }
}
exports.CycleCheckpointRadixDigest = CycleCheckpointRadixDigest;
//Represents a single bucket in the system
class CycleCheckpointBucket extends CheckpointData_1.CheckpointBucket {
    constructor(startTime, endTime, bucketID, validateData, updateData) {
        super(startTime, endTime, bucketID, validateData, updateData, CheckpointData_1.CheckpointType.Cycle);
    }
    async update(currentTime) {
        // Call parent update first
        super.update(currentTime);
    }
}
exports.CycleCheckpointBucket = CycleCheckpointBucket;
//Manages all buckets, routes incoming data to the correct bucket, and does periodic updates
class CycleCheckpointManager extends CheckpointData_1.CheckpointBucketManager {
    constructor() {
        const persistenceCallbacks = {
            validateData,
            updateData,
        };
        super(persistenceCallbacks, CheckpointData_1.CheckpointType.Cycle);
    }
    static getInstance() {
        if (!CycleCheckpointManager.instance) {
            CycleCheckpointManager.instance = new CycleCheckpointManager();
        }
        return CycleCheckpointManager.instance;
    }
}
//Represents a tally of all radix entries in the system
class CycleRadixDigestTally extends CheckpointData_1.RadixDigestTally {
    constructor(radix) {
        super(radix);
    }
}
exports.CycleRadixDigestTally = CycleRadixDigestTally;
// Define the validateData function
async function validateData(data) {
    const cycle = data.d;
    // Basic validation checks
    if (!cycle || cycle.counter === undefined || !cycle.cycleMarker || !cycle.cycleRecord) {
        Logger.mainLogger.error('Missing required cycle fields');
        return false;
    }
    // Validate cycle record fields
    if (!cycle.cycleRecord.start || cycle.cycleRecord.counter === undefined) {
        Logger.mainLogger.error('Invalid cycle record fields');
        return false;
    }
    // Verify timestamp matches cycle record start time
    if (data.t !== cycle.cycleRecord.start) {
        Logger.mainLogger.error('[CycleValidation] Timestamp mismatch with cycle record');
        return false;
    }
    // Verify address matches hash of cycle counter
    const expectedAddress = Crypto.hash(lib_types_1.Utils.safeStringify(cycle)).toLowerCase().substring(0, 2);
    if (data.a !== expectedAddress) {
        Logger.mainLogger.error('Address mismatch');
        return false;
    }
    // Verify hash matches data
    const calculatedHash = Crypto.hash(lib_types_1.Utils.safeStringify(cycle)).toLowerCase();
    if (calculatedHash !== data.h) {
        Logger.mainLogger.error('Hash mismatch');
        return false;
    }
    return true;
}
// Define the updateData function
async function updateData(data) {
    try {
        // Insert/Update into checkpointdata table
        const cycle = data.d;
        // average entry size  in ITN4: 18394.7144 bytes
        await (0, cycles_1.insertCycle)(cycle, false);
    }
    catch (err) {
        Logger.mainLogger.error('Failed to store cycle checkpoint data:', err);
        throw err;
    }
}
// Export the singleton instance
exports.cycleCheckpointManager = CycleCheckpointManager.getInstance();
//# sourceMappingURL=CycleData.js.map