"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryLatestOriginalTxs = exports.queryOriginalTxDataCountByCycles = exports.queryOriginalTxDataByTxId = exports.queryOriginalTxsData = exports.queryOriginalTxDataCount = exports.bulkInsertOriginalTxsData = exports.insertOriginalTxData = void 0;
// import { Signature } from 'shardus-crypto-types'
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
const OriginalTxsData_1 = require("../checkpoint/OriginalTxsData");
const checkpointStatus_1 = require("./checkpointStatus");
const State = require("../State");
async function insertOriginalTxData(originalTxData, storeCheckpoints = true) {
    try {
        if (storeCheckpoints && Config_1.config.checkpoint.bucketConfig.allowCheckpointUpdates) {
            // Create checkpoint for originalTxData
            const checkpointData = new OriginalTxsData_1.OriginalTxCheckpointData(originalTxData);
            const bucketID = (0, OriginalTxsData_1.calculateBucketID)(originalTxData);
            OriginalTxsData_1.originalTxCheckpointManager.addData(checkpointData, bucketID);
        }
        // Define the table columns based on schema
        const columns = ['txId', 'timestamp', 'cycle', 'originalTxData'];
        // Construct the SQL query with placeholders
        const placeholders = `(${columns.map(() => '?').join(', ')})`;
        const sql = `INSERT OR REPLACE INTO originalTxsData (${columns.join(', ')}) VALUES ${placeholders}`;
        // Map the `originalTxData` object to match the columns
        const values = columns.map((column) => typeof originalTxData[column] === 'object'
            ? (0, serialization_1.SerializeToJsonString)(originalTxData[column]) // Serialize objects to JSON
            : originalTxData[column]);
        // Execute the query directly (single-row insert)
        await db.run(_1.originalTxDataDatabase, sql, values);
        if (storeCheckpoints && Config_1.config.checkpoint.bucketConfig.allowCheckpointUpdates) {
            await (0, checkpointStatus_1.bulkUpdateCheckpointStatusField)(checkpointStatus_1.CheckpointStatusType.ORIGINAL_TX, true, undefined, undefined, [
                originalTxData.cycle,
            ]);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted OriginalTxData', originalTxData.txId);
        }
    }
    catch (err) {
        Logger.mainLogger.error(err);
        Logger.mainLogger.error('Unable to insert OriginalTxData or it is already stored in the database', originalTxData.txId);
    }
}
exports.insertOriginalTxData = insertOriginalTxData;
async function bulkInsertOriginalTxsData(originalTxsData, storeCheckpoints = true) {
    try {
        if (storeCheckpoints && Config_1.config.checkpoint.bucketConfig.allowCheckpointUpdates) {
            // Create checkpoints for all originalTxs
            for (const originalTx of originalTxsData) {
                const checkpointData = new OriginalTxsData_1.OriginalTxCheckpointData(originalTx);
                const bucketID = (0, OriginalTxsData_1.calculateBucketID)(originalTx);
                OriginalTxsData_1.originalTxCheckpointManager.addData(checkpointData, bucketID);
            }
        }
        // Then do the database operation
        const columns = ['txId', 'timestamp', 'cycle', 'originalTxData'];
        const placeholders = originalTxsData.map(() => `(${columns.map(() => '?').join(', ')})`).join(', ');
        const sql = `INSERT OR REPLACE INTO originalTxsData (${columns.join(', ')}) VALUES ${placeholders}`;
        // Flatten the `originalTxsData` array into a single list of values
        const values = originalTxsData.flatMap((txData) => columns.map((column) => typeof txData[column] === 'object'
            ? (0, serialization_1.SerializeToJsonString)(txData[column]) // Serialize objects to JSON
            : txData[column]));
        // Execute the single query for all originalTxsData
        await db.run(_1.originalTxDataDatabase, sql, values);
        if (storeCheckpoints && Config_1.config.checkpoint.bucketConfig.allowCheckpointUpdates) {
            const originalTxsDataToUpdate = originalTxsData.map((tx) => tx.cycle);
            await (0, checkpointStatus_1.bulkUpdateCheckpointStatusField)(checkpointStatus_1.CheckpointStatusType.ORIGINAL_TX, State.isSyncing, undefined, undefined, originalTxsDataToUpdate);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted OriginalTxsData', originalTxsData.length);
        }
    }
    catch (err) {
        Logger.mainLogger.error(err);
        Logger.mainLogger.error('Unable to bulk insert OriginalTxsData', originalTxsData.length);
    }
}
exports.bulkInsertOriginalTxsData = bulkInsertOriginalTxsData;
async function queryOriginalTxDataCount(startCycle, endCycle) {
    let originalTxsData;
    try {
        let sql = `SELECT COUNT(*) FROM originalTxsData`;
        const values = [];
        if (startCycle && endCycle) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        originalTxsData = await db.get(_1.originalTxDataDatabase, sql, values);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData count', originalTxsData);
    }
    return originalTxsData['COUNT(*)'] || 0;
}
exports.queryOriginalTxDataCount = queryOriginalTxDataCount;
async function queryOriginalTxsData(skip = 0, limit = 10, startCycle, endCycle) {
    let originalTxsData = [];
    if (!Number.isInteger(skip) || !Number.isInteger(limit)) {
        Logger.mainLogger.error('queryOriginalTxsData - Invalid skip or limit');
        return originalTxsData;
    }
    try {
        let sql = `SELECT * FROM originalTxsData`;
        const sqlSuffix = ` ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        const values = [];
        if (startCycle && endCycle) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        sql += sqlSuffix;
        originalTxsData = (await db.all(_1.originalTxDataDatabase, sql, values));
        originalTxsData.forEach((originalTxData) => {
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(originalTxData.originalTxData);
            // if (originalTxData.sign) originalTxData.sign = DeSerializeFromJsonString(originalTxData.sign)
        });
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData originalTxsData', originalTxsData);
    }
    return originalTxsData;
}
exports.queryOriginalTxsData = queryOriginalTxsData;
async function queryOriginalTxDataByTxId(txId, timestamp = 0) {
    try {
        const sql = `SELECT * FROM originalTxsData WHERE txId=?` + (timestamp ? ` AND timestamp=?` : '');
        const value = timestamp ? [txId, timestamp] : [txId];
        const originalTxData = (await db.get(_1.originalTxDataDatabase, sql, value));
        if (originalTxData) {
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(originalTxData.originalTxData);
            // if (originalTxData.sign) originalTxData.sign = DeSerializeFromJsonString(originalTxData.sign)
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('OriginalTxData txId', originalTxData);
        }
        return originalTxData;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryOriginalTxDataByTxId = queryOriginalTxDataByTxId;
async function queryOriginalTxDataCountByCycles(start, end) {
    const originalTxsDataCount = [];
    let dbOriginalTxsDataCount = [];
    try {
        const sql = `SELECT cycle, COUNT(*) FROM originalTxsData GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
        dbOriginalTxsDataCount = (await db.all(_1.originalTxDataDatabase, sql, [start, end]));
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData count by cycle', dbOriginalTxsDataCount);
    }
    if (dbOriginalTxsDataCount.length > 0) {
        for (let i = 0; i < dbOriginalTxsDataCount.length; i++) {
            /* eslint-disable security/detect-object-injection */
            originalTxsDataCount.push({
                cycle: dbOriginalTxsDataCount[i].cycle,
                originalTxDataCount: dbOriginalTxsDataCount[i]['COUNT(*)'],
            });
            /* eslint-enable security/detect-object-injection */
        }
    }
    return originalTxsDataCount;
}
exports.queryOriginalTxDataCountByCycles = queryOriginalTxDataCountByCycles;
async function queryLatestOriginalTxs(count) {
    if (!Number.isInteger(count)) {
        Logger.mainLogger.error('queryLatestOriginalTxs - Invalid count value');
        return null;
    }
    try {
        const sql = `SELECT * FROM originalTxsData ORDER BY cycle DESC, timestamp DESC LIMIT ${count ? count : 100}`;
        const originalTxsData = (await db.all(_1.originalTxDataDatabase, sql));
        if (originalTxsData.length > 0) {
            originalTxsData.forEach((tx) => {
                if (tx.originalTxData)
                    tx.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(tx.originalTxData);
                // if (tx.sign) tx.sign = DeSerializeFromJsonString(tx.sign)
            });
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Latest Original-Tx: ', originalTxsData);
        }
        return originalTxsData;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryLatestOriginalTxs = queryLatestOriginalTxs;
//# sourceMappingURL=originalTxsData.js.map