"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.allowedArchiversManager = void 0;
const path = require("path");
const fs = require("fs");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Logger = require("../Logger");
const ticketVerification_1 = require("../services/ticketVerification");
const security_1 = require("../types/security");
class AllowedArchiversManager {
    constructor() {
        this.currentConfig = null;
        this.configPath = '';
        this.isInitialized = false;
        this.globalAccountAllowedSigners = {};
        this.globalAccountMinSigRequired = 0;
    }
    initialize(configPath) {
        if (this.isInitialized) {
            return;
        }
        try {
            if (!configPath) {
                Logger.mainLogger.error('Config path is required');
                return;
            }
            this.configPath = path.resolve(configPath);
            if (!fs.existsSync(this.configPath)) {
                Logger.mainLogger.error('Config file does not exist');
                return;
            }
            // Load initial configuration
            this.loadAndVerifyConfig();
            // Watch for file changes
            fs.watchFile(this.configPath, { persistent: true }, (curr, prev) => {
                if (curr.mtime !== prev.mtime) {
                    this.loadAndVerifyConfig();
                }
            });
            this.isInitialized = true;
        }
        catch (error) {
            Logger.mainLogger.error('Failed to initialize AllowedArchiversManager:', error);
        }
    }
    stopWatching() {
        if (this.isInitialized && this.configPath) {
            fs.unwatchFile(this.configPath);
            this.isInitialized = false;
        }
    }
    setGlobalAccountConfig(allowedSigners, minSigRequired) {
        if (allowedSigners) {
            this.globalAccountAllowedSigners = allowedSigners;
        }
        if (minSigRequired >= 1) {
            this.globalAccountMinSigRequired = minSigRequired;
        }
        if (!allowedSigners && !minSigRequired) {
            return;
        }
        this.loadAndVerifyConfig(); // Reload config to apply changes
    }
    getArchiverWhitelistConfig() {
        try {
            if (!this.configPath) {
                Logger.mainLogger.error('Config path not set');
                return null;
            }
            const data = fs.readFileSync(this.configPath, 'utf8');
            const newConfig = lib_types_1.Utils.safeJsonParse(data);
            if (!this.validateConfig(newConfig)) {
                Logger.mainLogger.error('Invalid config structure');
                return null;
            }
            return {
                signatures: newConfig.signatures,
                allowedArchivers: newConfig.allowedArchivers,
            };
        }
        catch (error) {
            Logger.mainLogger.error('Failed to read configuration:', error);
            return null;
        }
    }
    validateConfig(config) {
        return !!(config && Array.isArray(config.allowedArchivers) && Array.isArray(config.signatures));
    }
    loadAndVerifyConfig() {
        try {
            const getArchiverConfig = this.getArchiverWhitelistConfig();
            if (!getArchiverConfig) {
                Logger.mainLogger.error('Failed to get archiver config');
                return;
            }
            const payload = {
                allowedArchivers: getArchiverConfig.allowedArchivers,
            };
            const isValidList = (0, ticketVerification_1.verifyMultiSigs)(payload, getArchiverConfig.signatures, this.globalAccountAllowedSigners, this.globalAccountMinSigRequired, security_1.DevSecurityLevel.HIGH);
            if (!isValidList.isValid) {
                Logger.mainLogger.error('Invalid signatures in new config');
                return;
            }
            this.currentConfig = getArchiverConfig;
        }
        catch (error) {
            Logger.mainLogger.error('Error loading/verifying config:', error);
        }
    }
    getCurrentConfig() {
        return this.currentConfig;
    }
    isArchiverAllowed(publicKey) {
        if (!publicKey || !this.currentConfig) {
            return false;
        }
        return this.currentConfig.allowedArchivers.some((archiver) => archiver.publicKey === publicKey);
    }
}
exports.allowedArchiversManager = new AllowedArchiversManager();
//# sourceMappingURL=allowedArchiversManager.js.map